# get_step_num
# get_step_states
# clear
# get_step_state
# set_state
# decrement_step
# increment_step

import os
import unittest
import shutil
import tempfile
import types
from mock import patch
from mock import Mock
from pyramid import testing
from pyramid.exceptions import ConfigurationError

class TestFormView(unittest.TestCase):
    def _getTargetClass(self):
        from pyramid_deform import FormView
        return FormView
        
    def _makeOne(self, request):
        klass = self._getTargetClass()
        inst = klass(request)
        return inst

    def test___call__show(self):
        schema = DummySchema()
        request = DummyRequest()
        inst = self._makeOne(request)
        inst.schema = schema
        inst.form_class = DummyForm
        result = inst()
        self.assertEqual(result, {'css_links': (), 'js_links': (),
                                  'form': 'rendered with None'})

    def test___call__show_with_appstruct(self):
        schema = DummySchema()
        request = DummyRequest()
        inst = self._makeOne(request)
        inst.schema = schema
        inst.form_class = DummyForm
        inst.appstruct = lambda: {'my': 'appstruct'}
        result = inst()
        self.assertEqual(result, {'css_links': (), 'js_links': (),
                                  'form': "rendered with {'my': 'appstruct'}"})

    def test___call__show_result_response(self):
        from webob import Response
        schema = DummySchema()
        request = DummyRequest()
        inst = self._makeOne(request)
        inst.schema = schema
        inst.form_class = DummyForm
        response = Response()
        inst.show = lambda *arg: response
        result = inst()
        self.assertEqual(result, response)

    def test___call__button_in_request(self):
        schema = DummySchema()
        request = DummyRequest()
        request.POST['submit'] = True
        inst = self._makeOne(request)
        inst.schema = schema
        inst.buttons = (DummyButton('submit'), )
        inst.submit_success = lambda *x: 'success'
        inst.form_class = DummyForm
        result = inst()
        self.assertEqual(result, 'success')
        
    def test___call__button_in_request_fail(self):
        schema = DummySchema()
        request = DummyRequest()
        request.POST['submit'] = True
        inst = self._makeOne(request)
        inst.schema = schema
        inst.buttons = (DummyButton('submit'), )
        import deform.exception
        def raiseit(*arg):
            raise deform.exception.ValidationFailure(None, None, None)
        inst.submit_success = raiseit
        inst.form_class = DummyForm
        inst.submit_failure = lambda *arg: 'failure'
        result = inst()
        self.assertEqual(result, 'failure')

    def test___call__button_in_request_fail_no_failure_handler(self):
        schema = DummySchema()
        request = DummyRequest()
        request.POST['submit'] = True
        inst = self._makeOne(request)
        inst.schema = schema
        inst.buttons = (DummyButton('submit'), )
        import deform.exception
        def raiseit(*arg):
            exc = deform.exception.ValidationFailure(None, None, None)
            exc.render = lambda *arg: 'failure'
            raise exc
        inst.submit_success = raiseit
        inst.form_class = DummyForm
        result = inst()
        self.assertEqual(result,
                         {'css_links': (), 'js_links': (), 'form': 'failure'})

    def test_get_bind_data_contains_request(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        data = inst.get_bind_data()
        self.assertTrue('request' in data)

    def test__call__binds_schema_with_get_bind_data(self):
        schema = DummySchema()
        request = DummyRequest()
        inst = self._makeOne(request)
        inst.schema = schema
        inst.form_class = DummyForm
        inst()
        # note: DummySchema sets kw to the bind data
        self.assertEqual(schema.kw, inst.get_bind_data())

    def test_form_options_applied(self):
        schema = DummySchema()
        request = DummyRequest()
        inst = self._makeOne(request)
        inst.schema = schema
        inst.form_class = DummyForm
        form_options = (('formid', 'custom-id'),
                        ('action', 'custom-action'),
                        ('method', 'GET'),
                        ('arbitrary-option', ''))
        inst.form_options = form_options

        def check_form(self, form):
            self.form = form
        inst.before = types.MethodType(check_form, inst)
        inst()
        form = inst.form
        # All options should end up on the form, overriding any defaults
        for key, value in dict(form_options).items():
            self.assertEqual(getattr(form, key), value)

class TestFormWizardView(unittest.TestCase):
    def _makeOne(self, wizard):
        from pyramid_deform import FormWizardView
        return FormWizardView(wizard)

    def test___call__step_zero_no_schemas(self):
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        result = inst(request)
        self.assertEqual(result, 'done')

    def test___call__step_zero_one_schema(self):
        schema = DummySchema()
        wizard = DummyFormWizard(schema)
        inst = self._makeOne(wizard)
        inst.form_view_class = DummyFormView
        request = DummyRequest()
        result = inst(request)
        self.assertEqual(result, 'viewed')

    def test___call__prev_not_ok(self):
        schema = DummySchema()
        schema.prev_ok = lambda *arg: False
        wizard = DummyFormWizard(schema)
        inst = self._makeOne(wizard)
        inst.form_view_class = DummyFormView
        request = DummyRequest()
        result = inst(request)
        self.assertEqual(result, 'viewed')

    def test___call__next_not_ok(self):
        schema = DummySchema()
        schema.next_ok = lambda *arg: False
        wizard = DummyFormWizard(schema)
        inst = self._makeOne(wizard)
        inst.form_view_class = DummyFormView
        request = DummyRequest()
        result = inst(request)
        self.assertEqual(result, 'viewed')

    def test_show(self):
        from pyramid_deform import WizardState
        form = DummyForm(None)
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.wizard_state = WizardState(request, 'name')
        result = inst.show(form)
        self.assertEqual(result, {'form': 'rendered with {}'})
        self.assertEqual(form.appstruct, {})

    def test_show_with_appstruct(self):
        from pyramid_deform import WizardState
        form = DummyForm(None)
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        class DummySchemaWithAppstruct(object):
            appstruct = {'1':'2'}
        inst.schema = DummySchemaWithAppstruct()
        request = DummyRequest()
        inst.request = request
        inst.wizard_state = WizardState(request, 'name')
        result = inst.show(form)
        self.assertEqual(result, {'form': "rendered with {'1': '2'}"})
        self.assertEqual(form.appstruct, {'1':'2'})

    def test_show_with_deserialize(self):
        from pyramid_deform import WizardState
        form = DummyForm(None)
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.schema = DummySchema()
        inst.schema.wizard_serializer = lambda *arg: DummySerializer('state2')
        inst.wizard_state = WizardState(request, 'name')
        result = inst.show(form)
        self.assertEqual(result, {'form': 'rendered with state2'})
        self.assertEqual(form.appstruct, 'state2')

    def test_next_success(self):
        from pyramid_deform import WizardState
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.wizard_state = WizardState(request, 'name')
        inst.schema = DummySchema()
        result = inst.next_success({'one':'one'})
        self.assertEqual(result.status, '302 Found')
        self.assertEqual(result.location, 'http://example.com')
        state = request.session['pyramid_deform.wizards']['name']
        self.assertEqual(state['step'], 1)
        self.assertEqual(state['states'][0], {'one':'one'})
        self.assertEqual(state['states']['schema'], {'one':'one'})

    def test_next_success_with_serializer(self):
        from pyramid_deform import WizardState
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.wizard_state = WizardState(request, 'name')
        inst.schema = DummySchema()
        inst.schema.wizard_serializer = lambda *arg: DummySerializer('state2')
        result = inst.next_success({'one':'one'})
        self.assertEqual(result.status, '302 Found')
        self.assertEqual(result.location, 'http://example.com')
        state = request.session['pyramid_deform.wizards']['name']
        self.assertEqual(state['step'], 1)
        self.assertEqual(state['states'][0], 'state2')
        self.assertEqual(state['states']['schema'], 'state2')

    def test_previous_success_at_step_zero(self):
        from pyramid_deform import WizardState
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.wizard_state = WizardState(request, 'name')
        inst.schema = DummySchema()
        result = inst.previous_success({'one':'one'})
        self.assertEqual(result.status, '302 Found')
        self.assertEqual(result.location, 'http://example.com')
        state = request.session['pyramid_deform.wizards']['name']
        self.assertEqual(state['states'][0], {'one':'one'})
        self.assertEqual(state['states']['schema'], {'one':'one'})
        self.assertFalse('step' in state)

    def test_previous_success_at_step_one(self):
        from pyramid_deform import WizardState
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.wizard_state = WizardState(request, 'name')
        inst.schema = DummySchema()
        states = inst.request.session['pyramid_deform.wizards'] = {}
        states['name'] = {'step':1}
        result = inst.previous_success({'one':'one'})
        self.assertEqual(result.status, '302 Found')
        self.assertEqual(result.location, 'http://example.com')
        state = request.session['pyramid_deform.wizards']['name']
        self.assertEqual(state['states'][1], {'one':'one'})
        self.assertEqual(state['states']['schema'], {'one':'one'})
        self.assertEqual(state['step'], 0)

    def test_previous_success_with_serializer(self):
        from pyramid_deform import WizardState
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.wizard_state = WizardState(request, 'name')
        inst.schema = DummySchema()
        inst.schema.wizard_serializer = lambda *arg:DummySerializer('state2')
        result = inst.previous_success({'one':'one'})
        self.assertEqual(result.status, '302 Found')
        self.assertEqual(result.location, 'http://example.com')
        state = request.session['pyramid_deform.wizards']['name']
        self.assertEqual(state['states'][0], 'state2')
        self.assertEqual(state['states']['schema'], 'state2')

    def test_previous_failure_at_step_zero(self):
        from pyramid_deform import WizardState
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.schema = DummySchema()
        inst.wizard_state = WizardState(request, 'name')
        result = inst.previous_failure(None)
        self.assertEqual(result.status, '302 Found')
        self.assertEqual(result.location, 'http://example.com')
        state = request.session['pyramid_deform.wizards']['name']
        self.assertFalse('step' in state)

    def test_previous_failure_at_step_one(self):
        from pyramid_deform import WizardState
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        request = DummyRequest()
        inst.request = request
        inst.schema = DummySchema()
        inst.wizard_state = WizardState(request, 'name')
        states = inst.request.session['pyramid_deform.wizards'] = {}
        states['name'] = {'step':1}
        result = inst.previous_failure(None)
        self.assertEqual(result.status, '302 Found')
        self.assertEqual(result.location, 'http://example.com')
        state = request.session['pyramid_deform.wizards']['name']
        self.assertEqual(state['step'], 0)

    def test_get_schema_serializer_no_serializer(self):
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        inst.schema = DummySchema()
        self.assertEqual(inst.get_schema_serializer(), None)

    def test_get_schema_serializer(self):
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        inst.schema = DummySchema()
        inst.schema.wizard_serializer = lambda schema: '123'
        self.assertEqual(inst.get_schema_serializer(), '123')

    def test_deserialize(self):
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        inst.schema = DummySchema()
        inst.schema.wizard_serializer = lambda *arg: DummySerializer('state2')
        self.assertEqual(inst.deserialize('state'), 'state2')
        
    def test_serialize(self):
        wizard = DummyFormWizard()
        inst = self._makeOne(wizard)
        inst.schema = DummySchema()
        inst.schema.wizard_serializer = lambda *arg: DummySerializer('state2')
        self.assertEqual(inst.serialize('state'), 'state2')
        
class TestWizardState(unittest.TestCase):
    def _makeOne(self, request):
        from pyramid_deform import WizardState
        return WizardState(request, 'name')

    def test__get_wizard_data_no_existing_data(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        data = inst._get_wizard_data()
        self.assertEqual(data, {})
        self.assertTrue('name' in request.session['pyramid_deform.wizards'])
        self.assertTrue(request.session._changed)

    def test__get_wizard_data_with_existing_data(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        state = {'abc':'123'}
        states = request.session['pyramid_deform.wizards'] = {}
        states['name'] = state
        inst.request = request
        data = inst._get_wizard_data()
        self.assertEqual(data, state)
        self.assertFalse(request.session._changed)

    def test_clear(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        state = {'abc':'123'}
        states = request.session['pyramid_deform.wizards'] = {}
        states['name'] = state
        inst.request = request
        inst.clear()
        self.assertEqual(request.session['pyramid_deform.wizards']['name'], {})

    def test_clear_get_step_num_from_params(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        request.GET['step'] = '1'
        inst.request = request
        self.assertEqual(inst.get_step_num(), 1)

    def test_clear_get_step_num_from_session(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        states = request.session['pyramid_deform.wizards'] = {}
        states['name'] = {'step':'1'}
        inst.request = request
        self.assertEqual(inst.get_step_num(), 1)

    def test_set_step_num(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        inst.request = request
        inst.set_step_num(5)
        self.assertEqual(inst.get_step_num(), 5)

    def test_get_step_states(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        states = request.session['pyramid_deform.wizards'] = {}
        states['name'] = {'states':'states', 'step':0}
        inst.request = request
        self.assertEqual(inst.get_step_states(), 'states')

    def test_get_step_state(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        states = request.session['pyramid_deform.wizards'] = {}
        states['name'] = {'states':{0:'state'}, 'step':0}
        inst.request = request
        self.assertEqual(inst.get_step_state(), 'state')

    def test_get_step_state_nondefault(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        request.session['pyramid_deform.wizards'] = {}
        inst.request = request
        self.assertEqual(inst.get_step_state('123'), '123')

    def test_set_step_state(self):
        request = DummyRequest()
        inst = self._makeOne(request)
        states = request.session['pyramid_deform.wizards'] = {}
        states['name'] = {'states':{0:'state'}, 'step':0}
        inst.request = request
        inst.set_step_state(0, 'schema', 'state2')
        self.assertEqual(states['name']['states'][0], 'state2')
        self.assertEqual(states['name']['states']['schema'], 'state2')

class TestFormWizard(unittest.TestCase):
    def _makeOne(self, name, done, *schemas):
        from pyramid_deform import FormWizard
        return FormWizard(name, done, *schemas)
    
    def test___call__(self):
        inst = self._makeOne('name', None, 'schema1', 'schema2')
        inst.form_wizard_view_class = DummyFormWizardView
        request = DummyRequest()
        result = inst(request)
        self.assertEqual(result.wizard, inst)

    def test_get_summary(self):
        schema1 = DummySchema()
        schema2 = DummySchema()
        inst = self._makeOne('name', None, schema1, schema2)
        request = DummyRequest()
        summary = inst.get_summary(request)
        self.assertEqual(
            summary,
            [{'current': True,
              'num': 0,
              'last': False,
              'name': 'schema',
              'title': 'title',
              'url': 'http://example.com?step=0',
              'first': True,
              'class': 'first hilight',
              'desc': 'desc'},
             {'current': False,
              'num': 1,
              'last': True,
              'name': 'schema',
              'title': 'title',
              'url': 'http://example.com?step=1',
              'first': False,
              'class': 'last',
              'desc': 'desc'}
             ])

class TestCRSFSchema(unittest.TestCase):
    def _getTargetClass(self):
        from pyramid_deform import CSRFSchema
        return CSRFSchema

    def _makeOne(self):
        return self._getTargetClass()()

    def test_validate_failure(self):
        from colander import Invalid
        inst = self._makeOne()
        request = DummyRequest()
        inst2 = inst.bind(request=request)
        self.assertRaises(Invalid, inst2.deserialize, {'csrf_token':''})

    def test_validate_missing(self):
        from colander import Invalid
        inst = self._makeOne()
        request = DummyRequest()
        inst2 = inst.bind(request=request)
        self.assertRaises(Invalid, inst2.deserialize, {})

    def test_validate_success(self):
        inst = self._makeOne()
        request = DummyRequest()
        inst2 = inst.bind(request=request)
        self.assertEqual(inst2.deserialize({'csrf_token':'csrf_token'}),
                         {'csrf_token': 'csrf_token'})


class TestSessionFileUploadTempStore(unittest.TestCase):
    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)
        
    def _getTargetClass(self):
        from . import SessionFileUploadTempStore
        return SessionFileUploadTempStore

    def _makeOne(self, request):
        return self._getTargetClass()(request)

    def _makeRequest(self):
        request = testing.DummyRequest()
        request.registry.settings = {}
        request.registry.settings['pyramid_deform.tempdir'] = self.tempdir
        request.session = DummySession()
        return request

    def test_no_tempdir_in_settings(self):
        request = testing.DummyRequest()
        request.registry.settings = {}
        self.assertRaises(ConfigurationError, self._makeOne, request)

    def test_preview_url(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        self.assertEqual(inst.preview_url(None), None)

    def test_contains_true(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        inst.tempstore['a'] = 1
        self.assertTrue('a' in inst)
        
    def test_contains_false(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        self.assertFalse('a' in inst)

    def test_setitem_stream_None(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        inst['a'] = {}
        self.assertEqual(inst.tempstore['a'], {})
        self.assertTrue(request.session._changed)

    def test_setitem_stream_file(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        here = os.path.dirname(__file__)
        thisfile = os.path.join(here, 'tests.py')
        fp = open(thisfile, 'rb')
        inst['a'] = {'fp':fp}
        self.assertTrue(inst.tempstore['a']['randid'])
        fn = os.path.join(self.tempdir, inst.tempstore['a']['randid'])
        with open(thisfile, 'rb') as f:
            expected = f.read()
        with open(fn, 'rb') as f:
            received = f.read()
        self.assertTrue(received, expected)
        self.assertTrue(request.session._changed)
        fp.close()
        inst['a']['fp'].close()
        
    def test_get_data_None(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        self.assertEqual(inst.get('a', True), True)

    def test_get_no_randid(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        inst.tempstore['a'] = {'fp':True}
        self.assertEqual(inst.get('a'), {'fp':True})

    def test_get_with_randid(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        fn = os.path.join(self.tempdir, '1234')
        with open(fn, 'wb') as f:
            f.write(b'abc')
        inst.tempstore['a'] = {'randid':'1234'}
        with open(fn, 'rb') as f:
            expected = f.read()
        with inst['a']['fp']  as f:
            received = f.read()
        self.assertEqual(received, expected)

    def test_get_with_randid_file_doesntexist(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        inst.tempstore['a'] = {'randid':'1234'}
        self.assertFalse('fp' in inst.get('a'))

    def test___getitem___notfound(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        self.assertRaises(KeyError, inst.__getitem__, 'a')
        
    def test___getitem___found(self):
        request = self._makeRequest()
        inst = self._makeOne(request)
        inst.tempstore['a'] = {}
        self.assertEqual(inst['a'], {})

class DummyForm(object):
    def __init__(self, schema, buttons=None, use_ajax=False, ajax_options='',
                 formid='deform', action='', method='POST', **kw):
        self.schema = schema
        self.buttons = buttons
        self.use_ajax = use_ajax
        self.ajax_options = ajax_options
        self.formid = formid
        self.action = action
        self.method = method
        self.__dict__.update(kw)

    def get_widget_resources(self):
        return {'js':(), 'css':()}

    def render(self, appstruct=None):
        self.appstruct = appstruct
        return 'rendered with {0}'.format(appstruct)

    def validate(self, controls):
        return 'validated'

class DummySchema(object):
    name = 'schema'
    description = 'desc'
    title = 'title'
    
    def bind(self, **kw):
        self.kw = kw
        return self
    
class DummyButton(object):
    def __init__(self, name):
        self.name = name
        
class DummyFormWizardView(object):
    def __init__(self, wizard):
        self.wizard = wizard

    def __call__(self, request):
        return self
    
class DummyFormWizard(object):
    name = 'name'
    def __init__(self, *schemas):
        self.schemas = schemas

    def done(self, request, states):
        return 'done'

class DummySession(dict):
    _changed = False
    def changed(self):
        self._changed = True

    def get_csrf_token(self):
        return 'csrf_token'

class DummyRequest(testing.DummyRequest):
    def __init__(self, *arg, **kw):
        testing.DummyRequest.__init__(self, *arg, **kw)
        self.session = DummySession()
    
class DummyFormView(object):
    def __init__(self, request):
        self.request = request

    def __call__(self):
        return 'viewed'
        
class DummySerializer(object):
    def __init__(self, result):
        self.result = result

    def deserialize(self, state):
        return self.result

    def serialize(self, state):
        return self.result

class TestConfigureZPTRenderer(unittest.TestCase):
    @patch('deform.form.Form')
    def test_translator(self, Form):
        from pyramid_deform import translator
        from pyramid_deform import configure_zpt_renderer

        configure_zpt_renderer()
        assert Form.default_renderer.translate is translator

    @patch('deform.form.Form')
    def test_search_path(self, Form):
        from pyramid_deform import configure_zpt_renderer

        search_path_before = ('search-path-before',)
        Form.default_renderer.loader.search_path = search_path_before
        configure_zpt_renderer(['deform:templates'])

        search_path = Form.default_renderer.loader.search_path
        assert len(Form.default_renderer.loader.search_path) == 2
        assert (search_path[-1],) == search_path_before
        assert search_path[0].endswith('deform' + os.path.sep + 'templates')

class TestIncludeMe(unittest.TestCase):
    @patch('pyramid_deform.configure_zpt_renderer')
    @patch('deform.form.Form')
    def test_default(self, Form, configure_zpt_renderer):
        from pyramid_deform import includeme

        config = Mock()
        config.registry.settings = {}
        includeme(config)

        assert config.add_translation_dirs.call_count == 1
        assert config.add_static_view.call_count == 1
        config.add_static_view.assert_called_with('static-deform', 'deform:static')
        configure_zpt_renderer.assert_called_with([])

    @patch('pyramid_deform.configure_zpt_renderer')
    @patch('deform.form.Form')
    def test_static_path(self, Form, configure_zpt_renderer):
        from pyramid_deform import includeme

        config = Mock()
        config.registry.settings = {
            'pyramid_deform.static_path': 'http://some.domain.com/override/path ', #also tests strip
            }
        includeme(config)

        assert config.add_translation_dirs.call_count == 1
        assert config.add_static_view.call_count == 1
        config.add_static_view.assert_called_with('http://some.domain.com/override/path', 'deform:static')
        configure_zpt_renderer.assert_called_with([])

    @patch('pyramid_deform.configure_zpt_renderer')
    @patch('deform.form.Form')
    def test_template_search_path(self, Form, configure_zpt_renderer):
        from pyramid_deform import includeme

        config = Mock()
        config.registry.settings = {
            'pyramid_deform.template_search_path': 'this-path',
            }
        includeme(config)

        assert config.add_translation_dirs.call_count == 1
        assert config.add_static_view.call_count == 1
        configure_zpt_renderer.assert_called_with(['this-path'])



import os
import binascii
import types

from pkg_resources import resource_filename

import colander
import deform
import deform.form
import deform.exception
import deform.widget
from deform.form import Button

from pyramid.exceptions import ConfigurationError
from pyramid.httpexceptions import HTTPFound
from pyramid.i18n import get_localizer
from pyramid.i18n import TranslationStringFactory
from pyramid.threadlocal import get_current_request
import sys


_ = TranslationStringFactory('pyramid_deform')

# True if we are running on Python 3.
PY3 = sys.version_info[0] == 3

if PY3: # pragma: no cover
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes
    long = int
else:
    string_types = basestring,
    integer_types = (int, long)
    class_types = (type, types.ClassType)
    text_type = unicode
    binary_type = str
    long = long

class FormView(object):
    """
    Helper view for Deform forms for use with the Pyramid framework.
    """
    #: Class object of the type of form to be created.
    #: Defaults to using the standard :class:`deform.form.Form` class.
    form_class = deform.form.Form

    #: Tuple of buttons or strings to pass to the form instance.
    #: Override in your derived class.
    buttons = ()

    #: `Colander` schema instance to be used to create the form instance.
    #: Provide your schema in your derived class.
    schema = None

    #: Two-tuple of options to pass as keyword arguments when instantiating
    #: the form instance of :attr:`form_class`. Any options that can be
    #: passed to this class' ``__init__`` can be provided here.
    form_options = ()

    def __init__(self, request):
        self.request = request

    def get_bind_data(self):
        """
        Return a ``dict``-like structure to bind to :attr:`schema`.

        By default, this method returns a ``dict`` with :attr:`request` -
        the object passed to :meth:`__init__`. The returned structure
        will be passed as keyword arguments to :meth:`schema.bind`.
        """
        return {'request': self.request}

    def __call__(self):
        """ 
        Prepares and render the form according to provided options.

        Upon receiving a ``POST`` request, this method will validate
        the request against the form instance. After validation, 
        this calls a method based upon the name of the button used for
        form submission and whether the validation succeeded or failed.
        If the button was named ``save``, then :meth:`save_success` will be
        called on successful validation or :meth:`save_failure` will
        be called upon failure. An exception to this is when no such
        ``save_failure`` method is present; in this case, the fallback
        is :meth:`failure``. 
        
        Returns a ``dict`` structure suitable for provision tog the given
        view. By default, this is the page template specified 
        """
        use_ajax = getattr(self, 'use_ajax', False)
        ajax_options = getattr(self, 'ajax_options', '{}')
        self.schema = self.schema.bind(**self.get_bind_data())
        form = self.form_class(self.schema, buttons=self.buttons,
                               use_ajax=use_ajax, ajax_options=ajax_options,
                               **dict(self.form_options))
        self.before(form)
        reqts = form.get_widget_resources()
        result = None

        for button in form.buttons:
            if button.name in self.request.POST:
                success_method = getattr(self, '%s_success' % button.name)
                try:
                    controls = self.request.POST.items()
                    validated = form.validate(controls)
                    result = success_method(validated)
                except deform.exception.ValidationFailure as e:
                    fail = getattr(self, '%s_failure' % button.name, None)
                    if fail is None:
                        fail = self.failure
                    result = fail(e)
                break

        if result is None:
            result = self.show(form)

        if isinstance(result, dict):
            result['js_links'] = reqts['js']
            result['css_links'] = reqts['css']

        return result

    def before(self, form):
        """
        Performs some processing on the ``form`` prior to rendering.
        
        By default, this method does nothing. Override this method
        in your dervived class to modify the ``form``. Your function
        will be executed immediately after instansiating the form
        instance in :meth:`__call__` (thus before obtaining widget resources,
        considering buttons, or rendering).
        """
        pass

    def appstruct(self):
        """
        Returns an ``appstruct`` for form default values when rendered.

        By default, this method does nothing. Override this method in
        your dervived class and return a suitable entity that can be
        used as an ``appstruct`` and passed to the
        :meth:`deform.Field.render` of an instance of
        :attr:`form_class`.
        """
        return None

    def failure(self, e):
        """
        Default action upon form validation failure.

        Returns the result of :meth:`render` of the given ``e`` object
        (an instance of :class:`deform.exception.ValidationFailure`) as the
        ``form`` key in a ``dict`` structure. 
        """
        return {
            'form': e.render(),
            }

    def show(self, form):
        """
        Render the given form, with or without an ``appstruct`` context.

        The given ``form`` argument will be rendered with an ``appstruct``
        if :meth:`appstruct` provides one.  Otherwise, it is rendered without.
        Returns the rendered form as the ``form`` key in a ``dict`` structure.
        """
        appstruct = self.appstruct()
        if appstruct is None:
            rendered = form.render()
        else:
            rendered = form.render(appstruct)
        return {
            'form': rendered,
            }

class WizardState(object):
    def __init__(self, request, wizard_name):
        self.wizard_name = wizard_name
        self.request = request

    def _get_wizard_data(self):
        session = self.request.session
        wizdatas = session.setdefault('pyramid_deform.wizards', {})
        wizdata = wizdatas.get(self.wizard_name, None)
        if wizdata is None:
            wizdata = {}
            wizdatas[self.wizard_name] = wizdata
            session.changed()
        return wizdata

    def clear(self):
        wizdata = self._get_wizard_data()
        wizdata.clear()
        self.request.session.changed()

    def get_step_num(self):
        step = self.request.GET.get('step')
        if step is not None:
            step = int(step)
            self.set_step_num(step)
        else:
            wizdata = self._get_wizard_data()
            step = wizdata.get('step', 0)
        return int(step)

    def set_step_num(self, num):
        wizdata = self._get_wizard_data()
        wizdata['step'] = num
        self.request.session.changed()

    def get_step_states(self):
        wizdata = self._get_wizard_data()
        states = wizdata.setdefault('states', {})
        return states

    def get_step_state(self, default=None):
        if default is None:
            default = {}
        states = self.get_step_states()
        step = self.get_step_num()
        return states.get(step, default)

    def set_step_state(self, num, name, state):
        states = self.get_step_states()
        states[num] = state
        states[name] = state
        self.request.session.changed()

    def decrement_step(self):
        step = self.get_step_num()
        if step > 0:
            self.set_step_num(step-1)

    def increment_step(self):
        step = self.get_step_num()
        self.set_step_num(step+1)

    def set_state(self, name, state):
        step = self.get_step_num()
        self.set_step_state(step, name, state)

class FormWizardView(object):

    form_view_class = FormView
    wizard_state_class = WizardState
    schema = None

    def __init__(self, wizard):
        self.wizard = wizard

    def __call__(self, request):
        self.request = request
        self.wizard_state = self.wizard_state_class(request, self.wizard.name)
        step = self.wizard_state.get_step_num()
        
        if step > len(self.wizard.schemas)-1:
            states = self.wizard_state.get_step_states()
            result = self.wizard.done(request, states)
            self.wizard_state.clear()
            return result
        form_view = self.form_view_class(request)
        schema = self.wizard.schemas[step]
        self.schema = schema.bind(request=request)
        form_view.schema = self.schema
        buttons = []

        prev_disabled = False
        next_disabled = False

        if hasattr(schema, 'prev_ok'):
            prev_disabled = not schema.prev_ok(request)

        if hasattr(schema, 'next_ok'):
            next_disabled = not schema.next_ok(request)

        prev_button = Button(name='previous', title='Previous',
                             disabled=prev_disabled)
        next_button = Button(name='next', title='Next',
                             disabled=next_disabled)
        done_button = Button(name='next', title='Done',
                             disabled=next_disabled)

        if step > 0:
            buttons.append(prev_button)

        if step < len(self.wizard.schemas)-1:
            buttons.append(next_button)
        else:
            buttons.append(done_button)

        form_view.buttons = buttons
        form_view.next_success = self.next_success
        form_view.previous_success = self.previous_success
        form_view.previous_failure = self.previous_failure
        form_view.show = self.show
        form_view.appstruct = getattr(schema, 'appstruct', None)
        result = form_view()
        return result

    def get_schema_serializer(self):
        serializer = getattr(self.schema, 'wizard_serializer', None)
        if serializer is not None:
            return serializer(self.schema)
        return None

    def deserialize(self, state):
        serializer = self.get_schema_serializer()
        if serializer is not None:
            state = serializer.deserialize(state)
        return state 

    def serialize(self, state):
        serializer = self.get_schema_serializer()
        if serializer is not None:
            state = serializer.serialize(state)
        return state

    def show(self, form):
        appstruct = getattr(self.schema, 'appstruct', None)
        state = self.wizard_state.get_step_state(appstruct)
        state = self.deserialize(state)
        result = dict(form=form.render(appstruct=state))
        return result

    def next_success(self, validated):
        validated = self.serialize(validated)
        self.wizard_state.set_state(self.schema.name, validated)
        self.wizard_state.increment_step()
        return HTTPFound(location = self.request.path_url)

    def previous_success(self, validated):
        validated = self.serialize(validated)
        self.wizard_state.set_state(self.schema.name, validated)
        self.wizard_state.decrement_step()
        return HTTPFound(location = self.request.path_url)

    def previous_failure(self, e):
        self.wizard_state.decrement_step()
        return HTTPFound(location = self.request.path_url)

class FormWizard(object):
    form_wizard_view_class = FormWizardView # for testing
    wizard_state_class = WizardState
    
    def __init__(self, name, done, *schemas):
        self.name = name
        self.done = done
        self.schemas = schemas

    def __call__(self, request):
        view = self.form_wizard_view_class(self)
        result = view(request)
        return result

    def get_summary(self, request):
        result = []
        state = WizardState(request, self.name)
        step = state.get_step_num()
        last = len(self.schemas) - 1
        for num, schema in enumerate(self.schemas):
            classes = []
            is_first = num == 0
            is_last = num == last
            is_current = num == step
            if is_first:
                classes.append('first')
            if is_last:
                classes.append('last')
            if is_current:
                classes.append('hilight')
            result.append({
                'num':num,
                'name':schema.name,
                'title':schema.title,
                'desc':schema.description,
                'current':step == num,
                'url':request.path_url + '?step=%s' % num,
                'first':is_first,
                'last':is_last,
                'class':' '.join(classes),
                })
        return result

@colander.deferred
def deferred_csrf_value(node, kw):
    return kw['request'].session.get_csrf_token()

@colander.deferred
def deferred_csrf_validator(node, kw):
    def csrf_validate(node, value):
        if value != kw['request'].session.get_csrf_token():
            raise colander.Invalid(node,
                                   _('Invalid cross-site scripting token'))
    return csrf_validate

class CSRFSchema(colander.Schema):
    """
    Schema base class which generates and validates a CSRF token
    automatically.  You must use it like so:

    .. code-block:: python

      from pyramid_deform import CSRFSchema
      import colander

      class MySchema(CSRFSchema):
          my_value = colander.SchemaNode(colander.String())

      And in your application code, *bind* the schema, passing the request
      as a keyword argument:

      .. code-block:: python

        def aview(request):
            schema = MySchema().bind(request=request)

      In order for the CRSFSchema to work, you must configure a *session
      factory* in your Pyramid application.
    """
    csrf_token = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.HiddenWidget(),
        default=deferred_csrf_value,
        validator=deferred_csrf_validator,
        )

def translator(term):
    request = get_current_request()
    if request is not None:
        return get_localizer(request).translate(term)
    else:
        return term.interpolate() if hasattr(term, 'interpolate') else term

def configure_zpt_renderer(search_path=()):
    default_paths = deform.form.Form.default_renderer.loader.search_path
    paths = []
    for path in search_path:
        pkg, resource_name = path.split(':')
        paths.append(resource_filename(pkg, resource_name))
    deform.form.Form.default_renderer = deform.ZPTRendererFactory(
        tuple(paths) + default_paths, translator=translator)

_marker = object()

class SessionFileUploadTempStore(object):
    def __init__(self, request):
        try:
            self.tempdir=request.registry.settings['pyramid_deform.tempdir']
        except KeyError:
            raise ConfigurationError(
                'To use SessionFileUploadTempStore, you must set a  '
                '"pyramid_deform.tempdir" key in your .ini settings. It '
                'points to a directory which will temporarily '
                'hold uploaded files when form validation fails.')
        self.request = request
        self.session = request.session
        self.tempstore = self.session.setdefault('substanced.tempstore', {})
        
    def preview_url(self, uid):
        return None

    def __contains__(self, name):
        return name in self.tempstore

    def __setitem__(self, name, data):
        newdata = data.copy()
        stream = newdata.pop('fp', None)

        if stream is not None:
            while True:
                randid = binascii.hexlify(os.urandom(20))
                if not isinstance(randid, string_types):
                    randid = randid.decode("ascii")
                fn = os.path.join(self.tempdir, randid)
                if not os.path.exists(fn):
                    # XXX race condition
                    fp = open(fn, 'w+b')
                    newdata['randid'] = randid
                    break
            for chunk in chunks(stream):
                fp.write(chunk)

        self.tempstore[name] = newdata
        self.session.changed()

    def get(self, name, default=None):
        data = self.tempstore.get(name)

        if data is None:
            return default

        newdata = data.copy()
            
        randid = newdata.get('randid')

        if randid is not None:

            fn = os.path.join(self.tempdir, randid)
            try:
                newdata['fp'] = open(fn, 'rb')
            except IOError:
                pass

        return newdata

    def __getitem__(self, name):
        data = self.get(name, _marker)
        if data is _marker:
            raise KeyError(name)
        return data

def chunks(stream, chunk_size=10000):
    while True:
        chunk = stream.read(chunk_size)
        if not chunk:
            break
        yield chunk


def includeme(config):
    """ Provide useful configuration to a Pyramid ``Configurator`` instance.

    Currently, this hook will set up and register translation paths
    for Deform and Colander, add a static view for Deform resources (uses
    ``pyramid_deform.static_path`` from the Pyramid configuration if
    specified else ``static-deform`` by default), and configures a
    template search path (if one is specified by
    ``pyramid_deform.template_search_path`` in your Pyramid
    configuration).
    """
    settings = config.registry.settings
    search_path = settings.get(
        'pyramid_deform.template_search_path', '').strip()

    config.add_translation_dirs('colander:locale', 'deform:locale')
    static_path = settings.get(
        'pyramid_deform.static_path', 'static-deform').strip()
    config.add_static_view(static_path, 'deform:static')

    configure_zpt_renderer(search_path.split())



##############################################################################
#
# Copyright (c) 2010 Agendaless Consulting and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

__version__ = '0.2'

import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
try:
    with open(os.path.join(here, 'README.txt')) as f:
        README = f.read()
    with open(os.path.join(here, 'CHANGES.txt')) as f:
        CHANGES = f.read()
except IOError:
    README = CHANGES = ''

install_requires = [
    'pyramid',
    'deform>=0.8.1' # button disabled arg
    ]

tests_require = ['nose', 'coverage', 'Mock']

docs_extras = ['Sphinx']

setup(name='pyramid_deform',
      version=__version__,
      description=('Bindings to the Deform form library for the Pyramid web '
                   'framework'),
      long_description=README + '\n\n' +  CHANGES,
      classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: Implementation :: CPython",
        "Framework :: Pyramid",
        ],
      keywords='web wsgi pyramid pylons deform forms form',
      author="Chris McDonough, Agendaless Consulting",
      author_email="pylons-discuss@googlegroups.com",
      url="http://docs.pylonshq.com",
      license="BSD-derived (http://www.repoze.org/LICENSE.txt)",
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      tests_require = tests_require,
      install_requires = install_requires,
      test_suite="pyramid_deform",
      extras_require = {
          'testing':tests_require,
          'docs':docs_extras,
          },
      entry_points = """\
      """,
      )
